// *****************************************************************************
//
// source:    AC_SVD_evnt.C: User code for VDS CnA part
//
// type:      source code
//
// created:   03 Nov 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    AC_SVD_evnt()
//
// called by: AC_evnt()
//
// hints:     see $HBROOT/ARTE/dev/include/dq/DQ.h for COMP_XYZ and LVL_ABC
//
// changes:
// 03 Nov 1999 (AG)  further developments
// 19 Nov 1999 (AG)  cna -> ac (alca)
// 22 Nov 1999 (AG)  includes in alca
// 10 Dez 1999 (MBr) user code added
// 04 Aug 2000 (MAP) filling of new alignment ntuple introduced
// 07 Aug 2000 (AG)  polished printout
// 09 Aug 2000 (AG)  removed unnecessary printout
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"
#include "grr.h"

// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "alca/AC_SVD.hh"
#include "alca/AC_vdsalign_hist.h"

#define VDSACLEN (RHP_MAXCOL/2) // max. entries in Ntuple...
unsigned int nvdsac   = 0;      // total number of hit/cogs so far written away...
int vdsaccum = 0;               // accepted rsegs so far
int vdstot   = 0;               // total VDS - rsegs
int vdsprocevt = 0;             // processed events...

// Quality-checks...
#define NCUT 7                  // hits/track >= 7
#define CHICUT 10               // chi2/track < 10
#define VIEWCUT 2               // hits/view >= 2
#define CLEANCUT 0.1            // window around hit which should be empty [cm]


//
// program code
//
int
AC_SVD_evnt()
{
  ARTE_mark( "AC_SVD_evnt 0", 0 );

  ++vdsprocevt;
  vdstot += distance(ArteTable<RSEG>::begin(),ArteTable<RSEG>::end());
  // shake hands...
  int rc = GRR_OK;
  while( rc == GRR_OK )
    rc = rhp_wait_for_one_msg_and_dispatch( NO_BLOCK );

  // check success of rhp-commands...
  int check = GRR_INVALID;

  //loop over rsegs
  for(ArteTableIterator<RSEG> rseg=ArteTable<RSEG>::begin();
      rseg!=ArteTable<RSEG>::end(); ++rseg) 
    { 
      //get only VDS-rsegs...
      if ( ((*rseg)->cmp != 1) && ((*rseg)->cmp != 65537) ) 
	continue;
      
      unsigned int tothits = distance ((*rseg)->hitb.begin(),(*rseg)->hitb.end());

      // stop filling info if we have collected enough...
      if ((nvdsac + tothits) > VDSACLEN ) 
	break;

      // Start Quality-checks...
      if (tothits < NCUT) continue; // hits per track OK?
      if ( (*rseg)->chi > CHICUT) continue; // chi2 OK?
      int nn = 0; // number of hits on n-side
      int np = 0; // number of hits on p-side
      bool clean = true; // CLEANCUT fullfilled?

      // loop over hits of VDS-rseg for quality-check
      for ( ArteRelationIterator<HITB> hitbit = (*rseg)->hitb.begin();
	    hitbit!= (*rseg)->hitb.end(); ++hitbit ) 
	{
	  int view = (*hitbit)->gede->lay;
	  if ( (view == 1) || (view == 4) ) ++nn; 
	  else if ( (view == 2) || (view == 3) ) ++np; 
	  else cout << "AC_SVD_evnt: Never should get here..." << endl;
	  // loop over the remaining hits on the same detector for CLEANCUT
	  for (ArteTable<HITB>::iterator hit = ArteTable<HITB>::begin(); 
	       hit != ArteTable<HITB>::end(); ++hit)
	    {
	      // skip wrong hits...
	      if ( ((*hit)->gede) != ((*hitbit)->gede) ) continue;
	      // skip hit on track
	      if ((*hit) == (*hitbit)) 
		continue;
	      if ( fabs((*hitbit)->x - (*hit)->x) < CLEANCUT)
		{
		  clean = false;
		  break;
		}
	    }//inner hit-loop
	  if (!clean) break;	
	}// outer hit loop (hits on rseg)
      if (!clean) continue; // CLEANCUT
      if ( (nn < VIEWCUT) || (np < VIEWCUT) ) continue; //VIEWCUT
      
      // Now we should have a VDS-RSEG with good quality => fill its info
      unsigned int nhits = 0; // hit-no. for this rseg
      ++ vdsaccum;
      // loop over hits of rseg...
      for ( ArteRelationIterator<HITB> hitbit = (*rseg)->hitb.begin();
	    hitbit!= (*rseg)->hitb.end(); ++hitbit ) 
	{
	  ++nhits;
	  // get the side-no.
	  ArtePointer<HITB> hitb(hitbit);     // hitb = pointer to HITB 
	  ArtePointer<GEDE> gede(hitb->gede); // gede = pointer to GEDE

	  char slname[5]; //safer than just sl#!!! sl# will always start with 1!
	  strncpy( slname, gede->gesl->name, 4 ); 
	  slname[4] = '\0';
	  int sl     = atoi( slname+2 );
	  // MBr 7-Bit id + 1 to avoid the 0 :)
	  int mbr = ((gede->lay)-1)+(4*((gede->sec)-1))+(16*(sl-1))+1;
	  double cog = (((*hitbit)->x - gede->xfw[0]) / gede->pit ); 
	  if (nhits==tothits || nhits==1) 
	    mbr = -mbr; // mark first & last hit of  track
  	  static float vec[2];
	  vec[0] = cog;
	  vec[1] = mbr;
	  if (nvdsac < VDSACLEN) // my little paranoia...
	    {
	      rhp_hfn(AC_ID(COMP_SVD,LVL_TST,ALIGN_DATA_NT),vec); 
	      ++nvdsac;
	    }
	}//hit-loop	    
    }//rseg-loop

  //cout << "AC_SVD_evnt: Wrote away " << nvdsac << " infos up to now." << endl;

  //publish and reset - don't forget to reset counters!
  
  check = rhp_publish(AC_ID(COMP_SVD,LVL_TST,ALIGN_DATA_NT));
  if (check == GRR_OK){
  }
  else if (check == GRR_INVALID){ 
    cout << "AC_SVD_evnt: ===========> WARNING: Couldn't publish ntuple - wrong ID" << endl;
  }
  // happens without a gatherer, e.g. offline!
  else {
    //AG cout << "AC_SVD_evnt: HUH? Nobody interested in my data? rc=" << rc << endl;
  }

  check = rhp_reset(AC_ID(COMP_SVD,LVL_TST,ALIGN_DATA_NT));
  if (check == GRR_OK){
  }
  else if (check == GRR_INVALID){ 
    cout << "AC_SVD_evnt: ===========> WARNING: Couldn't reset ntuple - wrong ID" << endl;
  }
  else if (check == GRR_ACCESS){
    //cout << "AC_SVD_evnt: ===========> WARNING: Couldn't reset ntuple - access denied!" << endl;
  }
  else {
    cout << "AC_SVD_evnt: HUH? This shouldn't be possible! rc=" << rc << endl;
  }

  nvdsac = 0;
  if ((vdsprocevt % 1000) == 0 ){
    cout << "AC_SVD_evnt: VDS-Alignment: Accepted so far " << vdsaccum << " out of " << vdstot 
	 << "AC_SVD_evnt: VDS-rsegs. => RATIO :" <<  (float)vdsaccum/(float)vdstot << endl;
  }

  ARTE_mark( "AC_SVD_evnt 1", 0 );
  return 0;
}
