#include "arte/arte.hh"
#include "grover/grover.hh"
#include "TROOT.h"        
#include "TApplication.h" 
#include "TCanvas.h"      
#include "TCONE.h"        
#include "TTUBE.h"        
#include "TBRIK.h"        
#include "TNode.h"        
#include "TRotMatrix.h"   


extern int xargc;    // # of command line arguments of main()
extern char** xargv; // list of -"-


int GroverDraw3D(GroverParticleList* Particles = 0,
		 GroverWireList*     Wires = 0,
		 GroverVertexList*   Primaries = 0,
		 GroverVertexList*   Secondaries = 0,
		 GroverVertexList*   MCPrimaries = 0,
		 GroverVertexList*   MCSecondaries = 0,
		 GroverParticleList* Specials = 0,
		 GroverCuts*         usercuts = 0)
{
  ///////////////////////////////////////////////////////////////
  //
  // GroverDraw3D
  // M.Sang 11 Oct 00
  //
  // Simple 3D event display function for Grover. 
  // Uses ROOT TGeometry classes and X3D interface. 
  // See http://root.cern.ch for more information.
  //
  // Modifications:
  //
  // 00-10-11 Max    First version - works OK, track drawing
  //                 disabled (still buggy and I'm away next week)
  //
  ///////////////////////////////////////////////////////////////


  // 1) Cuts
  static GroverCuts localcuts;
  GroverCuts* cuts = &localcuts;
  if (usercuts != 0) cuts = usercuts;
  bool verbose          = (cuts->Verbose == 1);
  bool drawprim         = (cuts->TDPrimaries == 1);
  bool drawsec          = (cuts->TDSecondaries == 1);
  bool drawwires        = (cuts->TDWires == 1);
  bool drawaxes         = (cuts->TDAxes == 1);
  bool drawtr           = (cuts->TDTargetRegion == 1);
  bool drawmcprim       = (cuts->TDMCPrimaries == 1);
  bool drawmcsec        = (cuts->TDMCSecondaries == 1);
  bool drawprimtracks   = (cuts->TDPrimaryTracks == 1);
  bool drawsectracks    = (cuts->TDSecondaryTracks == 1);
  bool drawunusedtracks = (cuts->TDUnusedTracks == 1);
  bool drawspecialtracks= (cuts->TDSpecialTracks == 1);
  if ( !(drawprim         || drawsec       || drawwires        || 
	 drawaxes         || drawmcprim    || drawmcsec        || 
	 drawprimtracks   || drawsectracks || drawunusedtracks || 
	 drawspecialtracks                                       )
      ) return 0;





  // 2) Constants
  const float nsig                 =   3.5; // number of sigma for box size
  const float SCALE                = 200.0; // for X3D step size
  const float PRIMARYTRACKLENGTH   = (cuts->TDPrimaryTrackLength);
  const float SECONDARYTRACKLENGTH = (cuts->TDSecondaryTrackLength);
  const float UNUSEDTRACKLENGTH    = (cuts->TDUnusedTrackLength);
  const float SPECIALTRACKLENGTH   = (cuts->TDSpecialTrackLength);
  const float XAXISLENGTH          = (cuts->TDXAxisLength);
  const float YAXISLENGTH          = (cuts->TDYAxisLength);
  const float ZAXISLENGTH          = (cuts->TDZAxisLength);
  const float AXISRADIUS           = (cuts->TDAxisRadius);
  const int   TARGETCOLOUR         = (cuts->TDWireColour);
  const int   PRIMARYCOLOUR        = (cuts->TDPrimaryColour);
  const int   SECONDARYCOLOUR      = (cuts->TDSecondaryColour);
  const int   MCPRIMARYCOLOUR      = (cuts->TDMCPrimaryColour);
  const int   MCSECONDARYCOLOUR    = (cuts->TDMCSecondaryColour);
  const int   PRIMARYTRACKCOLOUR   = (cuts->TDPrimaryColour);
  const int   SECONDARYTRACKCOLOUR = (cuts->TDSecondaryColour);
  const int   UNUSEDTRACKCOLOUR    = (cuts->TDUnusedTrackColour);
  const int   SPECIALTRACKCOLOUR   = (cuts->TDSpecialTrackColour);
  const int   TRACKSMOOTHNESS      = (cuts->TDTrackSmoothness);





  // 3) Initialisation
  static int eventNumber = 0;
  ++eventNumber;
  bool firstcall = (eventNumber == 1);
  static TApplication* GroverTApp;
  static TCanvas* GroverCanvas;
  if (firstcall) {
    cout << "*** GroverInit3D: Setting up ROOT/X3D display..." << endl;
    GroverTApp = new TApplication("GroverTApp",&xargc,xargv);
    GroverCanvas = 
      new TCanvas("GroverCanvas","Grover Vertex Display",200,10,700,500);
  }





  // 4) Define mother volume
  TBRIK Grover_tr("","","void",
		  XAXISLENGTH*SCALE/2.0,YAXISLENGTH*SCALE/2.0,ZAXISLENGTH*SCALE/2.0);
  Grover_tr.SetVisibility(((drawtr) ? 1 : 0));
  TNode MotherVolume("Mother","Mother volume",&Grover_tr);
  MotherVolume.cd(); // all other volumes contained in here now




  // 5)  Define volumes for target region and axis arrows
  TTUBE Grover_xaxis("","","void",
		     0,AXISRADIUS*SCALE,XAXISLENGTH*SCALE/2.0);
  TCONE Grover_xmrkr("", "","void",
		     0.2*SCALE,0.0,0.2*SCALE,0.0,0.0);
  TTUBE Grover_yaxis("","","void",
		     0,AXISRADIUS*SCALE,YAXISLENGTH*SCALE/2.0);
  TCONE Grover_ymrkr("", "","void",
		     0.2*SCALE,0.0,0.2*SCALE,0.0,0.0);
  TTUBE Grover_zaxis("","","void",
		     0,AXISRADIUS*SCALE,ZAXISLENGTH*SCALE/2.0);
  TCONE Grover_zmrkr("", "","void",
		     0.2*SCALE,0.0,0.2*SCALE,0.0,0.0);
  // cosmetic
  Grover_xaxis.SetNumberOfDivisions(4);
  Grover_yaxis.SetNumberOfDivisions(4);
  Grover_zaxis.SetNumberOfDivisions(4);
  Grover_xaxis.SetLineColor(1); Grover_xmrkr.SetLineColor(2);
  Grover_yaxis.SetLineColor(1); Grover_ymrkr.SetLineColor(3);
  Grover_zaxis.SetLineColor(1); Grover_zmrkr.SetLineColor(4);
  // rotation matrices
  Double_t ut1 = 90.0, ut2 = 90.0, ut3 = 0.0;
  Double_t up1 =  0.0, up2 = 90.0, up3 = 0.0;
  TRotMatrix Grover_zaxmat("Grover_zaxmat","Grover_zaxmat",
			   ut1,       up1,
			   ut2,       up2,
			   ut3,       up3);
  TRotMatrix Grover_xaxmat("Grover_xaxmat","Grover_xaxmat",
			   ut1+90.0,  up1,
			   ut2,       up2,
			   ut3+90.0,  up3);
  TRotMatrix Grover_yaxmat("Grover_yaxmat","Grover_yaxmat",
			   ut1+90.0,  up1,
			   ut2,       up2+90.0,
			   ut3+90.0,  up3+90.0);
  if (drawaxes) {
    TNode* Grover_node2 = new TNode("","",&Grover_zaxis,0,0,0,&Grover_zaxmat);
    TNode* Grover_node3 = new TNode("","",&Grover_xaxis,0,0,0,&Grover_xaxmat);
    TNode* Grover_node4 = new TNode("","",&Grover_yaxis,0,0,0,&Grover_yaxmat);
    TNode* Grover_node5 = new TNode("","",&Grover_zmrkr,0,0,ZAXISLENGTH*SCALE/2.0,&Grover_zaxmat);
    TNode* Grover_node6 = new TNode("","",&Grover_xmrkr,XAXISLENGTH*SCALE/2.0,0,0,&Grover_xaxmat);
    TNode* Grover_node7 = new TNode("","",&Grover_ymrkr,0,YAXISLENGTH*SCALE/2.0,0,&Grover_yaxmat);
  }


  
  // 6) Volumes for target wires
  int nodecounter = 0; // number of nodes
  char mname[40], sname[40], nname[40]; // matrix,shape,node names
  if (Wires != 0 && drawwires) {
    for (GroverWireList::iterator iwire = Wires->begin();
	 iwire != Wires->end(); ++iwire) {
      ++nodecounter;
      float  x  = iwire->X(),      y = iwire->Y(),      z = iwire->Z();
      float dx  = iwire->SizeX(), dy = iwire->SizeY(), dz = iwire->SizeZ();
      if (iwire->Number() == 2) {
	float swap = dy; dy=dz; dz=swap; // dirty hack for MC
      }
      sprintf(mname,"matrix%d",nodecounter);
      sprintf(sname,"shape%d",nodecounter);
      sprintf(nname,"node%d",nodecounter);
      TBRIK *shape = new TBRIK(sname,sname,"void",dx*SCALE/2.0,dy*SCALE/2.0,dz*SCALE/2.0);
      shape->SetLineColor(TARGETCOLOUR);
      TRotMatrix *matrix = new TRotMatrix(mname,mname,ut1,up1,ut2,up2,ut3,up3);
      TNode *node = new TNode(nname,nname,shape,
			      x*SCALE,y*SCALE,z*SCALE,matrix);        
    }
  }





  // 7) Volumes for primary vertices
  if (Primaries != 0 && drawprim) {
    for (GroverVertexList::iterator iprim = Primaries->begin();
	 iprim != Primaries->end(); ++iprim) {
      ++nodecounter;
      float x = iprim->X(), y = iprim->Y(), z = iprim->Z();
      float dx  = sqrt(fabs(iprim->Cov(0)));
      float dy  = sqrt(fabs(iprim->Cov(2)));
      float dz  = sqrt(fabs(iprim->Cov(5)));
      sprintf(sname,"shape%d",nodecounter);
      sprintf(nname,"node%d",nodecounter);
      TBRIK *shape = new TBRIK(sname,sname,"void",
			       dx*nsig*SCALE,
			       dy*nsig*SCALE,
			       dz*nsig*SCALE);
      shape->SetLineColor(PRIMARYCOLOUR);
      TNode *node = new TNode(nname,nname,shape,
			      x*SCALE,y*SCALE,z*SCALE,
			      &Grover_zaxmat);        
    }
  }





  // 8) Volumes for MC primary vertices
  if (MCPrimaries != 0 && drawmcprim) {
    for (GroverVertexList::iterator iprim = MCPrimaries->begin();
	 iprim != MCPrimaries->end(); ++iprim) {
      ++nodecounter;
      float x = iprim->X(), y = iprim->Y(), z = iprim->Z();
      float dx  = 0.005; // perhaps a little sphere instead?
      float dy  = 0.005;
      float dz  = 0.005;
      sprintf(sname,"shape%d",nodecounter);
      sprintf(nname,"node%d",nodecounter);
      TBRIK *shape = new TBRIK(sname,sname,"void",
			       dx*SCALE,
			       dy*SCALE,
			       dz*SCALE);
      shape->SetLineColor(MCPRIMARYCOLOUR);
      TNode *node = new TNode(nname,nname,shape,
			      x*SCALE,y*SCALE,z*SCALE,
			      &Grover_zaxmat);        
    }
  }




  // 9) Volumes for Primary tracks
  if (Primaries != 0 && drawprimtracks && false) {
    for (GroverVertexList::iterator iprim = Primaries->begin();
	 iprim != Primaries->end(); ++iprim) {
      for (GroverParticleList::iterator ipar = iprim->particles.begin();
	   ipar != iprim->particles.end(); ++ipar) {
	++nodecounter;
	float x = ipar->BeginX(), y = ipar->BeginY(), z = ipar->BeginZ();
	float dx2  = ipar->BeginCov(0);
	float dy2  = ipar->BeginCov(2);
	float dz   = PRIMARYTRACKLENGTH*SCALE;//fabs(ZAXISLENGTH - z);
	float d    = sqrt(fabs(dx2+dy2));
	sprintf(sname,"shape%d",nodecounter);
	sprintf(nname,"node%d",nodecounter);
	TTUBE *shape = new TTUBE(sname,sname,"void",
				 0.0, d*SCALE, dz*SCALE/2.0);
	shape->SetLineColor(PRIMARYTRACKCOLOUR);
	shape->SetNumberOfDivisions(TRACKSMOOTHNESS);
	float tx = ipar->BeginSlopeX();
	float ty = ipar->BeginSlopeY();
	TRotMatrix *matrix = new TRotMatrix(mname,mname,ut1+tx, up1,
					                ut2,    up2+ty,
					                ut3+tx, up3+ty);
	TNode *node = new TNode(nname,nname,shape,
				x*SCALE,y*SCALE,z*SCALE,
				matrix);        
      }
    }
  }



  // 10) Volumes for secondary vertices
  if (Secondaries != 0 && drawsec) {
    for (GroverVertexList::iterator isec = Secondaries->begin();
	 isec != Secondaries->end(); ++isec) {
      ++nodecounter;
      float x = isec->X(), y = isec->Y(), z = isec->Z();
      float dx  = sqrt(fabs(isec->Cov(0)));
      float dy  = sqrt(fabs(isec->Cov(2)));
      float dz  = sqrt(fabs(isec->Cov(5)));
      sprintf(sname,"shape%d",nodecounter);
      sprintf(nname,"node%d",nodecounter);
      TBRIK *shape = new TBRIK(sname,sname,"void",
			       dx*nsig*SCALE,
			       dy*nsig*SCALE,
			       dz*nsig*SCALE);
      shape->SetLineColor(SECONDARYCOLOUR);
      TNode *node = new TNode(nname,nname,shape,
			      x*SCALE,y*SCALE,z*SCALE,
			      &Grover_zaxmat); 
    }
  }



  // 11) Volumes for MC secondary vertices
  if (MCSecondaries != 0 && drawmcsec) {
    for (GroverVertexList::iterator isec = MCSecondaries->begin();
	 isec != MCSecondaries->end(); ++isec) {
      ++nodecounter;
      float x = isec->X(), y = isec->Y(), z = isec->Z();
      float dx  = 0.005; // perhaps a little sphere instead?
      float dy  = 0.005;
      float dz  = 0.005;
      sprintf(sname,"shape%d",nodecounter);
      sprintf(nname,"node%d",nodecounter);
      TBRIK *shape = new TBRIK(sname,sname,"void",
			       dx*SCALE,
			       dy*SCALE,
			       dz*SCALE);
      shape->SetLineColor(MCSECONDARYCOLOUR);
      TNode *node = new TNode(nname,nname,shape,
			      x*SCALE,y*SCALE,z*SCALE,
			      &Grover_zaxmat);        
    }
  }




  // 12) Volumes for secondary tracks
  if (Secondaries != 0 && drawsectracks && false) {
    for (GroverVertexList::iterator isec = Secondaries->begin();
	 isec != Secondaries->end(); ++isec) {
      for (GroverParticleList::iterator ipar = isec->particles.begin();
	   ipar != isec->particles.end(); ++ipar) {
	++nodecounter;
	float x = ipar->BeginX(), y = ipar->BeginY(), z = ipar->BeginZ();
	float dx2  = ipar->BeginCov(0);
	float dy2  = ipar->BeginCov(2);
	float dz   = SECONDARYTRACKLENGTH*SCALE;//fabs(ZAXISLENGTH - z);
	float d    = sqrt(fabs(dx2+dy2));
	sprintf(sname,"shape%d",nodecounter);
	sprintf(nname,"node%d",nodecounter);
	TTUBE *shape = new TTUBE(sname,sname,"void",
				 0.0, d*SCALE, dz*SCALE/2.0);
	shape->SetLineColor(SECONDARYTRACKCOLOUR);
	shape->SetNumberOfDivisions(TRACKSMOOTHNESS);
	float tx = ipar->BeginSlopeX();
	float ty = ipar->BeginSlopeY();
	TRotMatrix *matrix = 
	  new TRotMatrix(mname,mname,
			 ut1+(57.3*atan(tx)), up1,
			 ut2,                 up2+(57.3*atan(ty)),
			 ut3+(57.3*atan(tx)), up3+(57.3*atan(ty)));
	TNode *node = new TNode(nname,nname,shape,
				x*SCALE,y*SCALE,z*SCALE,
				matrix);        
      }
    }
  }




  // 13) Volumes for unused tracks
  if (Particles != 0 && drawunusedtracks && false) {
    for (GroverParticleList::iterator ipar = Particles->begin();
	 ipar != Particles->end(); ++ipar) {
      if (ipar->UsedInSecondary() != 0 ||
	  ipar->UsedInPrimary()   != 0   ) continue;
      ++nodecounter;
      ipar->PropagateBeginTo(-3.5);
      float x = ipar->BeginX(), y = ipar->BeginY(), z = ipar->BeginZ();
      float dx2  = ipar->BeginCov(0);
      float dy2  = ipar->BeginCov(2);
      float dz   = UNUSEDTRACKLENGTH*SCALE;//fabs(ZAXISLENGTH - z);
      float d    = sqrt(fabs(dx2+dy2));
      sprintf(sname,"shape%d",nodecounter);
      sprintf(nname,"node%d",nodecounter);
      TTUBE *shape = new TTUBE(sname,sname,"void",
			       0.0, d*SCALE, dz*SCALE/2.0);
      shape->SetLineColor(UNUSEDTRACKCOLOUR);
      shape->SetNumberOfDivisions(TRACKSMOOTHNESS);
      float tx = ipar->BeginSlopeX();
      float ty = ipar->BeginSlopeY();
      TRotMatrix *matrix = new TRotMatrix(mname,mname,ut1+tx, up1,
					              ut2,    up2+ty,
					              ut3+tx, up3+ty);
      TNode *node = new TNode(nname,nname,shape,
			      x*SCALE,y*SCALE,z*SCALE,
			      matrix);        
    }
  }





  // 14) Volumes for special tracks
  if (Specials != 0 && drawspecialtracks && false) {
    for (GroverParticleList::iterator ipar = Specials->begin();
	 ipar != Specials->end(); ++ipar) {
      ++nodecounter;
      float x = ipar->BeginX(), y = ipar->BeginY(), z = ipar->BeginZ();
      float dx2  = ipar->BeginCov(0);
      float dy2  = ipar->BeginCov(2);
      float dz   = SPECIALTRACKLENGTH*SCALE;//fabs(ZAXISLENGTH - z);
      float d    = sqrt(fabs(dx2+dy2));
      sprintf(sname,"shape%d",nodecounter);
      sprintf(nname,"node%d",nodecounter);
      TTUBE *shape = new TTUBE(sname,sname,"void",
			       0.0, d*SCALE, dz*SCALE/2.0);
      shape->SetLineColor(SPECIALTRACKCOLOUR);
      shape->SetNumberOfDivisions(TRACKSMOOTHNESS);
      float tx = ipar->BeginSlopeX();
      float ty = ipar->BeginSlopeY();
      TRotMatrix *matrix = new TRotMatrix(mname,mname,ut1+tx, up1,
					              ut2,    up2+ty,
					              ut3+tx, up3+ty);
      TNode *node = new TNode(nname,nname,shape,
			      x*SCALE,y*SCALE,z*SCALE,
			      matrix);        
    }
  }





  // 15) DRAW IT !!!
  MotherVolume.Draw();
  GroverCanvas->Update();
  if (verbose) cout << "*** GroverDraw3D: Mother volume has "
		    << nodecounter 
		    << " TNode daughters" << endl;
  if (verbose||true) cout << 
		       "*** GroverDraw3D: type 'm' in X3D window for menu"<<endl<<
		       "                       'q' to quit and return to ARTE" 
			  << endl;
  if (cuts->TDX3D == 1) GroverCanvas->x3d();

  return 0;
}
