#include "arte/arte.hh"
#include "grover/grover.hh"
#include "TRandom.h"       // ROOT random number generators
#include "arte/Chrono.hh"   // A timer

//
// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
// 00-09-20  Max      Skip very short MTRAs for improved speed
//

extern "C" { // for the *linker's* benefit!
  void CatsRefit(ArtePointer<RSEG>);
}

int GroverRefitRSEG(GroverCuts* usercuts = 0)
{

  /////////////////////////////////////////////////////////////////////
  //
  // Use momenta from MTRA to refit all VDS RSEGs using CatsRefit.
  // Momenta can be smeared via cuts
  //
  // M.Sang 7 Sep 00
  //
  /////////////////////////////////////////////////////////////////////

  const int MAX_RSEGS = 500;

  if (!ArteTable<MCIN>::booked()) {
    cout << "*** GroverRefitRSEG: ERROR - not Monte Carlo data!" <<endl;
    return -1;
  }


  // error flag
  int errorflag = 0;


  // cuts
  static GroverCuts temp;
  GroverCuts* cuts = &temp;
  if (usercuts != 0) cuts = usercuts;
  // To smear or not to smear?
  static bool  smearing    = (cuts->CheatSmearing == 1);
  // constant fractional error (if smearing)
  static float smear_const =  cuts->CheatSmearConst;
  // fractional error which is proportional to momentum (if smearing)
  static float smear_var   =  cuts->CheatSmearVar;   


  // just some variables
  static ArtePointer<RSEG> rsegs[MAX_RSEGS];
  int i = 0; 
  static TRandom* myRandom = new TRandom();


  // loop over mtra and get the rsegs corresponding to them
  for (ArteTable<MTRA>::iterator imtra = ArteTable<MTRA>::begin();
       imtra != ArteTable<MTRA>::end(); ++imtra) {
    if (imtra->leng < 3.0) continue; // MS
    rsegs[i] = RSEGCausedBy(imtra);
    if (rsegs[i] != 0) {
      float mom = sqrt(imtra->px*imtra->px+
		       imtra->py*imtra->py+
		       imtra->pz*imtra->pz );
      float inv_mom = 1.0/mom;
      if (smearing) {
	//
	// (dp/p)^2 = constcon^2 + (p*varcon)^2
	//
	// and the TDR says:
	// ITR: constcon = 0.0040, varcon = 0.00002
	// OTR: constcon = 0.0050, varcon = 0.00005
	// (so roughly constant until high momenta, ~ 50 GeV)
	//
	// expected fractional error (squared)
	float avg_corr_sq =     smear_const*smear_const +
	                    (mom*smear_var)*(mom*smear_var);
	// fractional errors for this track
	float constcon    =     myRandom->Gaus(0,smear_const);
	float varcon      = mom*myRandom->Gaus(0,smear_var);
	// smear
	float smeared_inv_mom = inv_mom*(1.0+constcon+varcon);
	rsegs[i]->pf = rsegs[i]->pe = smeared_inv_mom;
	// Variance of inverse momentum = (1/p * fractional error)^2
	// = avg_corr_sq/p^2
	rsegs[i]->cvf[14] = rsegs[i]->cve[14] = avg_corr_sq/(mom*mom);
	// and all off-diagonals to zero
	for (int kk=10;kk<14;kk++) rsegs[i]->cvf[kk] = 0.0;
	if (smeared_inv_mom <= 0.0) {
	  cout << "*** GroverRefitRSEG: ERROR - smeared momentum <= 0.0" << endl;
	  smeared_inv_mom = 0.0;
	  --errorflag;
	}
      }
      else { // not smearing - use momentum directly
	rsegs[i]->pf = rsegs[i]->pe = inv_mom;
	rsegs[i]->cvf[14] = rsegs[i]->cve[14] = 0.00001;
	for (int kk=10;kk<14;kk++) rsegs[i]->cvf[kk] = 0.0;
      }
      i++;
    }
  }


  // Refit the tracks
  for (int j=0; j<i; j++) CatsRefit(rsegs[j]);

  return errorflag;
}


