#ifndef _RCPOINT_HH_
#define _RCPOINT_HH_
//---------------------------------------------------------------------
// Created: 13.08.97,   Olya Igonkina

//---------------------------------------------------------------------


#include <iostream.h>
#include <vector>
#include <CLHEP/config/CLHEP.h>


#include <CLHEP/Vector/ThreeVector.h>
#include <CLHEP/Matrix/SymMatrix.h>
#include <CLHEP/Matrix/Matrix.h>

//#include "autoArteUtil.h"
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "marple/RcMatchFlag.hh"

extern HepDouble const MARPLE_INFINITY;
extern int const  c_matrix_pack[5][5];
extern int const  fortran_matrix_pack[5][5];

/** MARPLE base class of Reconstruction Points */

class RcPoint : public Hep3Vector
{
public:

  ///              default constructor
  inline RcPoint();

  ///              creates a RcPoint with default covariant matrix
  RcPoint( const Hep3Vector &rp,          
           HepDouble tx = 0.,             
           HepDouble ty = 0.,             
           HepDouble pe = 0.,             
	   int mark = 0);                   
  ///              creates a RcPoint with complete information 
  RcPoint( const Hep3Vector &rp,
	   HepDouble tx ,
	   HepDouble ty ,
	   HepDouble pe ,
	   const HepSymMatrix &cov5,
	   int mark = 0);
  ///              creates a RcPoint with momentum equals 0
  RcPoint( const Hep3Vector &rp,
           HepDouble tx ,
           HepDouble ty ,
      	   const HepSymMatrix &cov,
	   int mark = 0);
  ///              take array of double precision as covariant matrix 
  RcPoint( const Hep3Vector &rp,
           HepDouble tx ,
           HepDouble ty ,
           HepDouble pe ,
      	   const double cov_array[15],
	   int mark = 0);
  ///              take array of single precision as covariant matrix 
  RcPoint( const Hep3Vector &rp,
           HepDouble tx ,
           HepDouble ty ,
           HepDouble pe ,
      	   const float cov_array[15],
	   int mark = 0);
  ///              the same with momentum equals 0, double precision
  RcPoint( const Hep3Vector &rp,
           HepDouble tx ,
           HepDouble ty ,
      	   const double cov_array[15],
	   int mark = 0);
  ///              the same with momentum equals 0, single precision
  RcPoint( const Hep3Vector &rp,
           HepDouble tx ,
           HepDouble ty ,
      	   const float cov_array[15],
	   int mark = 0);

  ///               copy constructor 
  inline RcPoint( const RcPoint &rp );  

  ///               destructor
  virtual ~RcPoint();

  ///                             return value of iPar parameter, iPar = 1-5
  inline HepDouble    var( int iPar) const;

  ///                             return tx = px/pz
  inline HepDouble     tx() const;  
  ///                             return ty = py/pz
  inline HepDouble     ty() const;
  ///                             return pe = Q/p (charge over momentum)
  inline HepDouble     pe() const;
  ///                             return covariant matrix
  inline const HepSymMatrix & cov() const;
  ///                             return marking flag of RcPoint
  inline int           mark() const;

  ///                             px / p
  inline HepDouble     cosTx() const;   
  ///                             py / p
  inline HepDouble     cosTy() const;   
  ///                             pz / p
  inline HepDouble     cosTz() const;

  ///                             set value of iPar parameter, iPar = 1-5
  inline void          setvar( int iPar, HepDouble var);

  ///                             set tx = px/pz
  inline void          settx(  HepDouble tx) ;  
  ///                             set ty = py/pz
  inline void          setty(  HepDouble ty) ;
  ///                             set pe = Q/p (charge over momentum)
  inline void          setpe(  HepDouble pe) ;
  ///                             set covariant matrix
  inline void          setcov( const HepSymMatrix &cov) ;
  ///                     put value as (i,j) element of covariant matrix, i,j=1,5
  inline void          setcov( int i, int j, HepDouble value) ;
  ///                     put covariant matrix as array of double precision
  inline void          setcov( const double cov_array[15]);
  ///                     put covariant matrix as array of single precision
  inline void          setcov( const float cov_array[15]);
  ///                     mark RcPoint
  inline void          setmark(int mark);

  ///equal 1 if all diagonal elements of cov() are not zero, otherwise 0
  int covMatrixOk() const;

  ///set diagonal elements of cov() to MARPLE\_INFINITY instead of 0 
  int checkAndCorrectCov();
  
  ///                   assigment operator
  inline RcPoint & operator = (const RcPoint &rp );

  ///                   return relations with MTRA 
  virtual const std::vector<ArtePointer<MTRA> >  mtraList() const; 

  ///                   return relation with HITB bank
  virtual const std::vector<ArtePointer<HITB> >  hitbList() const; 

  ///                   return relation with HITC bank
  virtual const std::vector<ArtePointer<HITC> >  hitcList() const; 

  // update ARTE bank connected with point
  virtual void updateARTE(int & iErr ) const;

  /// prolongate to z\_next position, CHANGE RcPoint! 
  void prolongate ( HepDouble z_next );

  //if linear = 1 - linear approximation is used, 
  //imsGesl number of superlayer where the search for closest plane starts 

  /// prolongate to z\_next position and return error message, CHANGE RcPoint! 
  void prolongate ( HepDouble z_next,
		    int & iErr,
		    int linear = 0,
		    int imsGesl = 0 );     

  // void prolongate2 ( HepDouble z_next,
  //	     int & iErr,
  //	     int linear = 0 );

  ///swim  the RcPoint to z\_next without Multiple Scattering, use linear approximation if pe() equals zero
  void fastswim ( HepDouble z_next, int & iErr);  
  ///                               use linear approximation
  void lineswim ( HepDouble z_next );    
  ///                               use Runge-Kutta method
  void swimRunge( HepDouble z_next , 
		  int & iErr,
		  HepDouble FFstep = 0 ); 

  ///correct covariant matrix by Multiple Scattering angle 
  void addMs( const HepDouble ms);

  /// correct  momentum by Energy Loss, set fromTargetToMuon = 1 if the particle is traced from the target to MUON detector, set 0 or -1 wise-versa
  void correctEnergyLoss( const HepDouble eLoss, int fromTargetToMuon );

  /// Kalman filter to update information with measured value HITB ( CHANGE RcPoint), return $\Delta \chi^2$ of procedure. Note, that link to HITB ARTE table should be controlled by derived class
  virtual HepDouble filter( ArtePointer< HITB> hitb, int & iErr );
  
  /// Kalman refit of all HITB connected, start from the smallest Z ( or bigest one if this end is closer to z() ),return $\chi^2$ and recalculate parameters
  virtual HepDouble refit(int & iErr);
  /// Kalman refit from z() to the bigest value Z of connected HITB 
  virtual HepDouble refitForward(int & iErr);
  /// Kalman refit from z() to the smallest value Z of connected HITB 
  virtual HepDouble refitBack(int & iErr);
  
  static int dummy ;

  ///estimate momentum using target information and correct it (CHANGE RcPoint), the number of iteration is less than nMaxIter
  void estimatePEfromTAR( HepDouble tarX, HepDouble tarY, HepDouble tarZ,
			  HepDouble covTarXX, HepDouble covTarYY, 
			  HepDouble covTarXY = 0,
			  int nMaxIter = 6, 
			  int & iErr = dummy,
			  int coordinate = 1); // 0 if slopes are taken as tarX
 
  ///estimate momentum using GTAR information and correct it (CHANGE RcPoint), the number of iteration is less than nMaxIter
  void estimatePEfromTAR( int nIter = 6, 
			  int & iErr = dummy);
  
  //estimate momentum using another RcPoint information and correct it (CHANGE given RcPoint), the number of iterations is less than nMaxIter. It is not recomendet for usage!!!
  void estimatePEfromPoint( const RcPoint & measure,
			    RcMatchFlag chooseFlag,
			    int nMaxIter, 
			    int & iErr);

  
  // return message: "MARPLE: impossible to invert covariance matrix"
  inline void error_message() const;


protected:

  // function which perform linear propogation of MS matrix correction
  // use to speed up process
  void lineMsCorr( unsigned int iGeslMin, 
		   unsigned int iGeslMax ,
		   HepDouble zend,
		   int forward = 1); 

  ///estimate parameter {\it iPar} using target information and correct it(CHANGE RcPoint), the number of iteration is less than nMaxIter
  void estimatePARfromTAR( int iPar, 
			   HepDouble tarX, HepDouble tarY, HepDouble tarZ,
			   HepDouble covTarXX, HepDouble covTarYY, 
			   HepDouble covTarXY = 0,
			   int nMaxIter = 6, 
			   int & iErr = dummy,
			   int coordinates = 1);  // 0 if slopes are taken as tarX
 
  ///estimate parameter {\it iPar} using GTAR information and correct it(CHANGE RcPoint), the number of iteration is less than nMaxIter
  void estimatePARfromTAR( int iPar, 
			   int nMaxIter = 5, 
			   int & iErr = dummy);


  ///estimate parameter {\it iPar} using another RcPoint and correct it(CHANGE given RcPoint), the number of iteration is less than nMaxIter
  void estimatePARfromPoint( int iPar,
			     const RcPoint & measure,
			     RcMatchFlag chooseFlag,
			     int nMaxIter, 
			     int & iErr);
  
  // calculate the distance from point to the end of superlayer GESL
  inline HepDouble correct_zforward ( HepDouble z_point, 
				      ArtePointer<GESL> igesl );  
  inline HepDouble correct_zback    ( HepDouble z_point, 
				      ArtePointer<GESL> igesl );  

private:
  // + Hep3Vector fvec3
  ///               slope tx = px/pz 
  HepDouble  ftx; 
  ///               slope ty = py/pz 
  HepDouble  fty; 
  ///               slope pe = Q/p (charge over momentum)
  HepDouble  fpe;  
  ///               covariant matrix (5,5)
  HepSymMatrix fcov5;
  ///  marking flag, it could be changed by RcMatchBuilder::doMatch()
  int fmark;  
 
};

// ostream operator to print RcPoint information
ostream & operator << (ostream &, const RcPoint &);

//---------------------------------------------------------------------
//some function which used by RcPoint class
//---------------------------------------------------------------------

// ostream operator to print MTRA information
ostream & operator << (ostream &, const std::vector<ArtePointer<MTRA> > & );

///return row in MTRA table if at least one element in mtraList1 == any element in mtraList2, in case there are several of them returns first one in mtraList1
ArtePointer<MTRA> 
RcMtraPointer( const std::vector<ArtePointer<MTRA> > & mtraList1, 
	       const std::vector<ArtePointer<MTRA> > & mtraList2 );
/// return 1 if at least one element in mtraList1 == any element in mtraList2
int RcMtraOk( const std::vector<ArtePointer<MTRA> > & mtraList1, 
	      const std::vector<ArtePointer<MTRA> > & mtraList2 );

///return 1 in MTRA table if mtraEntry == one of the elements in mtraList
int RcMtraOk( ArtePointer<MTRA>  & mtraEntry, 
	      const std::vector<ArtePointer<MTRA> > & mtraList );

///return row in MTRA table if mtraEntry == one of the elements in mtraList
ArtePointer<MTRA> RcMtraPointer( ArtePointer<MTRA>  & mtraEntry, 
				 const std::vector<ArtePointer<MTRA> > & 
				 mtraList);

//return 1 if RcPoint leaved by lepton from J/psi
int RcFromJpsi( const RcPoint &);

#include "marple/RcPoint.icc"
#endif
