/* Usage:

   call_via_istr_3par              (F f, (Arg1*)0, ..., (ArgN*)0,
                                    (P1*)0, (P2*)0, (P3*)0); or
   call_via_istr_check_EOF_3par    (F f, (Arg1*)0, ..., (ArgN*)0,
                                    (P1*)0, (P2*)0, (P3*)0); or
   call_via_istr_conditionally_3par(F f, (Arg1*)0, ..., (ArgN*)0,
                                    (P1*)0, (P2*)0, (P3*)0, Condition c);

   where F f - some function object which takes N+3 (0<=N<=2) arguments and
   expression f(a1, .., aN, p1, p2, p3)
   (a1, .., aN are of types Arg1, .., ArgN respectively,
   p1, p2, p3 are of types P1, P2, P3)
   is valid.
   If call_*_conditionally_3par(..) is used: Condition c is such an object
   that the expression c(is) is valid ("is" is of type istream*).

   call_via_istr*(...) returns an object of type CallViaIstr0..5_3Par<...>
   which has the operator()(istream* is, P1 p1, P2 p2, P3 p3) =>
   it extracts N arguments from "*is",
   if they are extracted successfully ("*is" state is good), "Condition c"
   is applied to "is" and:
   if ( c(is) ) => function f(a1, .., aN, p1, p2, p3) is called.
   In case of simple call_via_istr_3par(..) version condition is substituted
   by the object which always returns true, so that condition has no effect;
	 call_via_istr_check_EOF_3par(..) has a condition, which checks that after
	 extracting of all arguments (*is) is empty.
*/

// Author V.Balagura (balagura@mail.desy.de)   (16.04.98).

#ifndef _call_via_istr_3par_hh
#define _call_via_istr_3par_hh 1

#include "ptset/call_via_istr.hh"

/* -------------------- 0 args -------------------- */
template <class F, class P1, class P2, class P3, class Condition>
class CallViaIstr0_3Par {
public:
  F function;
  Condition condition;

  CallViaIstr0_3Par(F f, Condition c = Condition())
	 : function(f), condition(c) {}
  void operator()(istream* is, P1 p1, P2 p2, P3 p3) {
    if ( condition(is) ) function(p1, p2, p3);
  }
};
template <class F, class P1, class P2, class P3, class Condition>
inline CallViaIstr0_3Par<F, P1, P2, P3, Condition>
call_via_istr_conditionally_3par(F f, P1*, P2*, P3*, Condition c) {
  return CallViaIstr0_3Par<F, P1, P2, P3, Condition>(f, c);
}
/* -------------------- 1 arg -------------------- */
template <class F, class A1, class P1, class P2, class P3, class Condition>
class CallViaIstr1_3Par {
public:
  F function;
  Condition condition;

  CallViaIstr1_3Par(F f, Condition c = Condition())
	 : function(f), condition(c) {}
  void operator()(istream* is, P1 p1, P2 p2, P3 p3) {
    A1 a1;
    *is >> a1;
    if ( !(*is) ) { global_error()("can not extract argument"); return; }
    if ( condition(is) ) function(a1, p1, p2, p3);
    else global_error() << a1;
  }
};
template <class F, class A1, class P1, class P2, class P3, class Condition>
inline CallViaIstr1_3Par<F, A1, P1, P2, P3, Condition>
call_via_istr_conditionally_3par
(F f, A1*, P1*, P2*, P3*, Condition c) {
  return CallViaIstr1_3Par<F, A1, P1, P2, P3, Condition>(f, c);
}
/* -------------------- 2 args -------------------- */
template <class F, class A1, class A2, class P1, class P2, class P3,
class Condition>
class CallViaIstr2_3Par {
public:
  F function;
  Condition condition;

  CallViaIstr2_3Par(F f, Condition c = Condition())
	 : function(f), condition(c) {}
  void operator()(istream* is, P1 p1, P2 p2, P3 p3) {
    A1 a1;
    A2 a2;
    *is >> a1;
    if ( !(*is) ) { global_error()("can not extract first argument"); return; }
    *is >> a2;
    if ( !(*is) )
      { global_error()("can not extract second argument"); global_error() << a1; return; }
    if ( condition(is) ) function(a1, a2, p1, p2, p3);
    else global_error() << a2 << a1;
  }
};
template <class F, class A1, class A2, class P1, class P2, class P3,
class Condition>
inline CallViaIstr2_3Par<F, A1, A2, P1, P2, P3, Condition>
call_via_istr_conditionally_3par
(F f, A1*, A2*, P1*, P2*, P3*, Condition c) {
  return CallViaIstr2_3Par<F, A1, A2, P1, P2, P3, Condition>(f, c);
}

/* -------------------- call_via_istr_3par -------------------- */
/* No condition on istream is required */

template <class F, class P1, class P2, class P3>
inline CallViaIstr0_3Par<F, P1, P2, P3, _IstrOk>
call_via_istr_3par(F f, P1*, P2*, P3*) {
  return CallViaIstr0_3Par<F, P1, P2, P3, _IstrOk>(f, _IstrOk());
}
template <class F, class A1, class P1, class P2, class P3>
inline CallViaIstr1_3Par<F, A1, P1, P2, P3, _IstrOk>
call_via_istr_3par(F f, A1*, P1*, P2*, P3*) {
  return CallViaIstr1_3Par<F, A1, P1, P2, P3, _IstrOk>(f, _IstrOk());
}
template <class F, class A1, class A2, class P1, class P2, class P3>
inline CallViaIstr2_3Par<F, A1, A2, P1, P2, P3, _IstrOk>
call_via_istr_3par(F f, A1*, A2*, P1*, P2*, P3*) {
  return CallViaIstr2_3Par<F, A1, A2, P1, P2, P3, _IstrOk>(f, _IstrOk());
}

/* ----------------- call_via_istr_check_EOF_3par ----------------- */
/* Condition: istream is empty */

template <class F, class P1, class P2, class P3>
inline CallViaIstr0_3Par<F, P1, P2, P3, _CheckIstr_EOF>
call_via_istr_check_EOF_3par(F f, P1*, P2*, P3*) {
  return CallViaIstr0_3Par<F, P1, P2, P3, _CheckIstr_EOF>
	(f, _CheckIstr_EOF());
}
template <class F, class A1, class P1, class P2, class P3>
inline CallViaIstr1_3Par<F, A1, P1, P2, P3, _CheckIstr_EOF>
call_via_istr_check_EOF_3par(F f, A1*, P1*, P2*, P3*) {
  return CallViaIstr1_3Par<F, A1, P1, P2, P3, _CheckIstr_EOF>
	(f, _CheckIstr_EOF());
}
template <class F, class A1, class A2, class P1, class P2, class P3>
inline CallViaIstr2_3Par<F, A1, A2, P1, P2, P3, _CheckIstr_EOF>
call_via_istr_check_EOF_3par(F f, A1*, A2*, P1*, P2*, P3*) {
  return CallViaIstr2_3Par<F, A1, A2, P1, P2, P3, _CheckIstr_EOF>
	  (f, _CheckIstr_EOF());
}

#endif
