/* This file can be used to create initialized vectors "on the fly".
   
   E.g. if you have some function
     f(const vector<string>& names);
   and you'd like to pass an array of names to it ("a", "bb", "ccc")
   you may write:
     f(init_vector("a", "bb", "ccc"));
   This is equivalent to:
     vector<string> names(3);
     names[0] = "a"; names[1] = "bb"; names[2] = "ccc";
     f(names);

   init_vector(...) can accept from 1 to 6 arguments of any type T (all
   arguments should be of the same type). It just constructs
   vector<T>, fills it with the arguments you supplied (vector size == 1 ... 6
   depending on the number of arguments) and returns it.

   If you'd like to construct the function accepting varying number of
   arguments, but all arguments have the same type
   (e.g. your function g(...) should handle calls
     g("only one argument");
     g("first", "second argument");
     g("a", "bb", "ccc", "dddd");
     etc.)
   you may define f(const vector<string>& args);
   and call
     f(init_vector("only one argument"));
     f(init_vector("first", "second argument"));
     f(init_vector("a", "bb", "ccc", "dddd"));
   this is the way to avoid C-like calls of functions with not known number
   of arguments (but in case when all arguments are of the same type).
*/

// Author V.Balagura (balagura@mail.desy.de)   (11.01.2000).

#ifndef init_vector_h
#define init_vector_h 1

#include <string>
#include <vector>

template <class T>
vector<T> init_vector(const T& t0) {
  vector<T> v(1); v[0]=t0; return v;
}

template <class T>
vector<T> init_vector(const T& t0, const T& t1) {
  vector<T> v(2); v[0]=t0; v[1]=t1; return v;
}

template <class T>
vector<T> init_vector(const T& t0, const T& t1, const T& t2) {
  vector<T> v(3); v[0]=t0; v[1]=t1; v[2]=t2; return v;
}

template <class T>
vector<T> init_vector(const T& t0, const T& t1, const T& t2, const T& t3) {
  vector<T> v(4); v[0]=t0; v[1]=t1; v[2]=t2; v[3]=t3; return v;
}

template <class T>
vector<T> init_vector(const T& t0, const T& t1, const T& t2, const T& t3,
		      const T& t4) {
  vector<T> v(5); v[0]=t0; v[1]=t1; v[2]=t2; v[3]=t3; v[4]=t4; return v;
}

template <class T>
vector<T> init_vector(const T& t0, const T& t1, const T& t2, const T& t3,
		      const T& t4, const T& t5) {
  vector<T> v(6); v[0]=t0; v[1]=t1; v[2]=t2; v[3]=t3; v[4]=t4; v[5]=t5;
  return v;
}

/* --------------- For C++ strings --------------- */
inline vector<string> init_vector(char t0[]) {
  vector<string> v(1); v[0]=t0; return v;
}
inline vector<string> init_vector(char t0[], char t1[]) {
  vector<string> v(2); v[0]=t0; v[1]=t1; return v;
}
inline vector<string> init_vector(char t0[], char t1[], char t2[]) {
  vector<string> v(3); v[0]=t0; v[1]=t1; v[2]=t2; return v;
}
inline vector<string> init_vector(char t0[], char t1[], char t2[], char t3[]) {
  vector<string> v(4); v[0]=t0; v[1]=t1; v[2]=t2; v[3]=t3; return v;
}
inline vector<string> init_vector(char t0[], char t1[], char t2[], char t3[],
			   char t4[]) {
  vector<string> v(5); v[0]=t0; v[1]=t1; v[2]=t2; v[3]=t3; v[4]=t4; return v;
}
inline vector<string> init_vector(char t0[], char t1[], char t2[], char t3[],
			   char t4[], char t5[]) {
  vector<string> v(6); v[0]=t0; v[1]=t1; v[2]=t2; v[3]=t3; v[4]=t4; v[5]=t5;
  return v;
}

#endif
