// *****************************************************************************
//
// source:    Selection of D mesons using the following channels:
//               D0 :  K- pi+
//               ~D0:  K+ pi-
//               D+ :  2pi+ K-
//               D- :  2pi- K+
//             Background monitoring: pi- K-, pi- pi+ K+
//             Returns:
//               1 upon success, 0 otherwise  
//
// dependences: RTRA bank filled 
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 04 Aug 2000 (DD) D+/- added
// 06 Aug 2000 (DD) quality histograms
// 15 Aug 2000 (DD) Background check added
// 26 Aug 2000 (DD) Variable mass limit
// 06 Sep 2000 (DD) File description added
//
// *****************************************************************************

#include "reco/rcX.h"
#include "reco/rcXkinema.h"
#include "reco/rcXsegment.h"
#include "rhp.h"
#include "dq/DQ.h"


rcX *rcD0_MC1=0;
rcX *rcD0_MC2=0;
rcX *rcD0_RC1=0;
rcX *rcD0_RC2=0;
rcX *rcDm_RC3=0;
rcX *rcDp_RC4=0;
rcX *rcD0_BG1=0;
rcX *rcDp_BG2=0;



#ifdef USE_ROOT_DD
#define DM_LO 1.5
#define DM_HI 2.5
#else
#define DM_LO 1.5
#define DM_HI 2.5
#endif

int D0__Piplus_Kminus_MC();
int D0__Piminus_Kplus_MC();
int D0__Piplus_Kminus_RC();
int D0__Piminus_Kplus_RC();
int Dminus__Piminus_Piminus_Kplus_RC();
int Dplus__Piplus_Piplus_Kminus_RC();
int maximizeRTRA();

//
//  Main D selection
//
int 
D0()
{
  // -- kill old reconstruction that might be saved in DST/MINI
  static int toppD0=kinema::findTopp(421).index();     
  segment::killRTRA(3,toppD0);
  static int toppD0bar=kinema::findTopp(-421).index(); 
  segment::killRTRA(3,toppD0bar);
  static int toppDplus=kinema::findTopp(411).index();     
  segment::killRTRA(3,toppDplus);
  static int toppDminus=kinema::findTopp(-411).index(); 
  segment::killRTRA(3,toppDminus);

  D0__Piplus_Kminus_RC();
  D0__Piminus_Kplus_RC();
  Dminus__Piminus_Piminus_Kplus_RC();
  Dplus__Piplus_Piplus_Kminus_RC();
  return 0;
}




/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////  REAL RECO   /////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

int
D0__Piplus_Kminus_RC()
{
  int verb=0;
  
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  D0_RC1 <<<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcD0_RC1) {
    cout<<"initializing D0 a"<<endl;
    rcD0_RC1=new rcX("rec_D0a", "D0 -> Pi+K-", verb);
    decay d0;
    d0.rlevel        = 0;         // 0=full, 1=semi, 2=mtra
    d0.fit           = 203;       // = 'long' decay
    d0.book_tables   = 1;         // 1/0
    d0.root_output   = 1;         // make root file or not
    d0.verb          = verb;      // printout level
    rcD0_RC1->setMother(d0);    // set
    vertexAttrib vd0;
    vd0.rlevel            = 2;     // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vd0.topp              = kinema::findTopp(421); // pointer to mother
    vd0.max_vertex_chi2   = 5;  // limit for vertex acceptance 
    vd0.verb              = verb;  // printout level
    vd0.min_mass          = DM_LO; // minimum mass
    vd0.max_mass          = DM_HI; // maximum mass
    vd0.allowed_v0_tracks = 1;     // maximum # of tracks assigned to v0
    vd0.change_track_rtra = 0;     // change track after mass refit
    vd0.min_pt2_of_tracks = 0.25;  // minimum sum of pt^2 with respect to decay direction
    vd0.max_pt2_of_tracks = 1.60;  // maximum sum of pt^2 with respect to decay direction
    //vd0.max_drho_from_wire= 0.025; // maximum impact parameter at z of primary vertex
    vd0.min_dz_from_wire  = 0.15;  // minimum decay length
    rcD0_RC1->setVertex(vd0);      // set

    attribute pi1;
    pi1.topp         = kinema::findTopp(211);
    pi1.mass         = pi1.topp->mass;
    pi1.minP         = 0;
    pi1.maxP         = 200;
    pi1.minHIO       = 15;
    pi1.minHSI       = 6;
    pi1.minHRI       = 1;
    pi1.minPt        = 0.15;
//     pi1.minLRPI      = 0.1;
    pi1.minLREMUPI   = 0.1;
    pi1.maxEoP       = 0.8;
    pi1.maxCandidate = 10;
    pi1.useVdsOnce   = 1;
    pi1.useMainOnce  = 1;
//     pi1.maxChi       = 500;
//     pi1.lund         = 211; // MC
//     pi1.mother       = 421; // MC
    rcD0_RC1->addDaughter(pi1);

    attribute k1;
    k1.topp         = kinema::findTopp(-321);
    k1.mass         = k1.topp->mass;
    k1.minP         = -200;
    k1.maxP         = 0;
    k1.minHIO       = 15;
    k1.minHSI       = 6;
    k1.minHRI       = 1;
    k1.minPt        = 0.3;
    k1.minLRK       = 0.1; 
    k1.maxLRPI      = 0.01;
    k1.maxEoP       = 0.8;
    k1.maxCandidate = 10;
    k1.useVdsOnce   = 1;
    k1.useMainOnce  = 1;
//     k1.maxChi       = 500;
//     k1.lund         = -321; // MC
//     k1.mother       = 421;  // MC
    rcD0_RC1->addDaughter(k1);

    if (verb) rcD0_RC1->printDaughters();
  }
  rcD0_RC1->rc();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  D0_RC1 <<<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}

//
//
//
int
D0__Piminus_Kplus_RC()
{
  int verb=0;
  
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  D0_RC2 <<<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcD0_RC2) {
    cout<<"initializing D0 b"<<endl;
    rcD0_RC2=new rcX("rec_D0b", "D0 -> Pi-K", verb);
    decay d0;
    d0.rlevel        = 0;         // 0=full, 1=semi, 2=mtra
    d0.fit           = 203;       // = 'long' decay
    d0.book_tables   = 1;         // 1/0
    d0.root_output   = 1;         // make root file or not
    d0.verb          = verb;      // printout level
    rcD0_RC2->setMother(d0);      // set
    vertexAttrib vd0;
    vd0.rlevel            = 2;     // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vd0.topp              = kinema::findTopp(-421); // pointer to mother
    vd0.max_vertex_chi2   = 5;  // limit for vertex acceptance 
    vd0.verb              = verb;  // printout level
    vd0.min_mass          = DM_LO; // minimum mass
    vd0.max_mass          = DM_HI; // maximum mass
    vd0.allowed_v0_tracks = 1;     // max. # of tracks assigned to v0
    vd0.change_track_rtra = 0;     // change rtra after mass refit
    vd0.min_pt2_of_tracks = 0.25;  // minimum sum of pt^2 with respect to decay direction
    vd0.max_pt2_of_tracks = 1.60;  // maximum sum of pt^2 with respect to decay direction
    //vd0.max_drho_from_wire= 0.025; // maximum impact parameter at z of primary vertex
    vd0.min_dz_from_wire  = 0.15;  // minimum decay length
    rcD0_RC2->setVertex(vd0);      // set

    attribute pi1;
    pi1.topp         = kinema::findTopp(-211);
    pi1.mass         = pi1.topp->mass;
    pi1.minP         = -200;
    pi1.maxP         = 0;
    pi1.minHIO       = 15;
    pi1.minHSI       = 6;
    pi1.minHRI       = 1;
    pi1.minPt        = 0.15;
//     pi1.minLRPI      = 0.1; 
    pi1.minLREMUPI   = 0.1;
    pi1.maxEoP       = 0.8;
    pi1.maxCandidate = 10;
    pi1.useVdsOnce   = 1;
    pi1.useMainOnce  = 1;
//     pi1.maxChi       = 500;
//     pi1.lund         = -211; // MC
//     pi1.mother       = -421; // MC
    rcD0_RC2->addDaughter(pi1);

    attribute k1;
    k1.topp         = kinema::findTopp(321);
    k1.mass         = k1.topp->mass;
    k1.minP         = 0;
    k1.maxP         = 200;
    k1.minHIO       = 15;
    k1.minHSI       = 6;
    k1.minHRI       = 1;
    k1.minPt        = 0.3;
    k1.minLRK       = 0.1; 
    k1.maxLRPI      = 0.01;
    k1.maxEoP       = 0.8;
    k1.maxCandidate = 10;
    k1.useVdsOnce   = 1;
    k1.useMainOnce  = 1;
//     k1.maxChi       = 500;
//     k1.lund         = 321;  // MC
//     k1.mother       = -421; // MC
    rcD0_RC2->addDaughter(k1);

    if (verb) rcD0_RC2->printDaughters();
  }
  rcD0_RC2->rc();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  D0_RC2 <<<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}


//
//
//
int
Dminus__Piminus_Piminus_Kplus_RC()
{
  int verb=0;
  
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  Dm_RC3 <<<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcDm_RC3) {
    cout<<"initializing Dminus"<<endl;
    rcDm_RC3=new rcX("rec_Dm", "D- -> 2Pi-K", verb);
    decay dm;
    dm.rlevel        = 0;         // 0=full, 1=semi, 2=mtra
    dm.fit           = 203;       // = 'long' decay
    dm.book_tables   = 1;         // 1/0
    dm.root_output   = 1;         // make root file or not
    dm.verb          = verb;      // printout level
    rcDm_RC3->setMother(dm);      // set
    vertexAttrib vdm;
    vdm.rlevel            = 2;       // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vdm.topp              = kinema::findTopp(-411); // pointer to mother
    vdm.max_vertex_chi2   = 5;       // limit for vertex acceptance 
    vdm.verb              = verb;    // printout level
    vdm.min_mass          = DM_LO;   // minimum mass
    vdm.max_mass          = DM_HI;   // maximum mass
//     vdm.allowed_v0_tracks = 0;       // max. # tracks assigned to v0
    vdm.change_track_rtra = 0;       // change track parameters after mass fit
    vdm.min_dz_from_wire  = 0.15;    // minimum decay length
    rcDm_RC3->setVertex(vdm);        // set

    attribute pi1;
    pi1.topp         = kinema::findTopp(-211);
    pi1.mass         = pi1.topp->mass;
    pi1.clone        = 2; // 2 x pi-
    pi1.minP         = -200;
    pi1.maxP         = 0;
    pi1.minHIO       = 15;
    pi1.minHSI       = 6;
    pi1.minHRI       = 1;
    pi1.minPt        = 0.15;
//     pi1.minLRPI      = 0.1;
    pi1.minLREMUPI   = 0.1;
    pi1.maxEoP       = 0.8;
    pi1.maxCandidate = 10;
    pi1.useVdsOnce   = 1;
    pi1.useMainOnce  = 1;
//     pi1.maxChi       = 500;
//     pi1.lund         = -211; // MC id
//     pi1.mother       = -411; // MC id
    rcDm_RC3->addDaughter(pi1);

    attribute k1;
    k1.topp         = kinema::findTopp(321);
    k1.mass         = k1.topp->mass;
    k1.minP         = 0;
    k1.maxP         = 200;
    k1.minHIO       = 15;
    k1.minHSI       = 6;
    k1.minHRI       = 1;
    k1.minPt        = 0.3;
    k1.minLRK       = 0.1;
    k1.maxEoP       = 0.8;
//     k1.maxLRPI      = 0.01;
    k1.maxCandidate = 10;
    k1.useVdsOnce   = 1;
    k1.useMainOnce  = 1;
//     k1.maxChi       = 500;
//     k1.lund         = 321;  // MC id
//     k1.mother       = -411; // MC id
    rcDm_RC3->addDaughter(k1);

    if (verb) rcDm_RC3->printDaughters();
  }
  rcDm_RC3->rc();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  Dm_RC3 <<<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}


int
Dplus__Piplus_Piplus_Kminus_RC()
{
  int verb=0;
  
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  Dp_RC4 <<<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcDp_RC4) {
    cout<<"initializing Dplus"<<endl;
    rcDp_RC4=new rcX("rec_Dp", "D+ -> 2Pi+K-", verb);
    decay dp;
    dp.rlevel        = 0;         // 0=full, 1=semi, 2=mtra
    dp.fit           = 203;       // = 'long' decay
    dp.book_tables   = 1;         // 1/0
    dp.root_output   = 1;         // make root file or not
    dp.verb          = verb;      // printout level
    rcDp_RC4->setMother(dp);      // set
    vertexAttrib vdp;
    vdp.rlevel            = 2;     // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vdp.topp              = kinema::findTopp(411); // pointer to mother
    vdp.max_vertex_chi2   = 5;     // limit for vertex acceptance
    vdp.verb              = verb;  // printout level
    vdp.min_mass          = DM_LO; // minimum mass
    vdp.max_mass          = DM_HI; // maximum mass
//     vdp.allowed_v0_tracks = 0;     // max. # tracks assigned to v0
    vdp.change_track_rtra = 0;     // change track parameters after mass fit
    vdp.min_dz_from_wire  = 0.15;  // minimum decay length
    rcDp_RC4->setVertex(vdp);      // set

    attribute pi1;
    pi1.topp         = kinema::findTopp(211);
    pi1.mass         = pi1.topp->mass;
    pi1.clone        = 2;
    pi1.minP         = 0;
    pi1.maxP         = 200;
    pi1.minHIO       = 15;
    pi1.minHSI       = 6;
    pi1.minHRI       = 1;
    pi1.minPt        = 0.15;
//     pi1.minLRPI      = 0.1;
    pi1.minLREMUPI   = 0.1;
    pi1.maxEoP       = 0.8;
    pi1.maxCandidate = 10;
    pi1.useVdsOnce   = 1;
    pi1.useMainOnce  = 1;
//     pi1.maxChi       = 500;
//     pi1.lund         = 211; // MC
//     pi1.mother       = 411; // MC
    rcDp_RC4->addDaughter(pi1);

    attribute k1;
    k1.topp         = kinema::findTopp(-321);
    k1.mass         = k1.topp->mass;
    k1.minP         = -200;
    k1.maxP         = 0;
    k1.minHIO       = 15;
    k1.minHSI       = 6;
    k1.minHRI       = 1;
    k1.minPt        = 0.3;
    k1.minLRK       = 0.1;
    k1.maxEoP       = 0.8;
//     k1.maxLRPI      = 0.01; 
    k1.maxCandidate = 10;
    k1.useVdsOnce   = 1;
    k1.useMainOnce  = 1;
//     k1.maxChi       = 500;
//     k1.lund         = -321; // MC
//     k1.mother       = 411;  // MC
    rcDp_RC4->addDaughter(k1);

    if (verb) rcDp_RC4->printDaughters();
  }
  rcDp_RC4->rc();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  Dp_RC4 <<<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}


/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////  BACKGROUND  /////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

int
D0__Piminus_Kminus_BG()
{

  int verb=0;
  
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  D0_BG1 <<<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcD0_BG1) {
    cout<<"initializing D0 a"<<endl;
    rcD0_BG1=new rcX("bg_D0", "D0 -> Pi-K-", verb);
    decay d0;
    d0.rlevel        = 0;         // 0=full, 1=semi, 2=mtra
    d0.fit           = 103;       // = 'long' decay
    d0.book_tables   = 1;         // 1/0
    d0.root_output   = 1;         // make root file or not
    d0.verb          = verb;      // printout level
    rcD0_BG1->setMother(d0);      // set
    vertexAttrib vd0;
    vd0.rlevel           = 2;     // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vd0.topp             = kinema::findTopp(421); // pointer to mother
    vd0.max_vertex_chi2  = 10.0;  // limit for vertex acceptance 
    vd0.verb             = verb;  // printout level
    vd0.min_mass         = DM_LO; // minimum mass
    vd0.max_mass         = DM_HI; // maximum mass
    vd0.allowed_v0_tracks = 1;    // max # tracks assigned to v0
    rcD0_BG1->setVertex(vd0);     // set

    attribute pi1;
    pi1.topp         = kinema::findTopp(211);
    pi1.mass         = pi1.topp->mass;
    pi1.minP         = -200;
    pi1.maxP         = 0;
    pi1.minPt        = 0.15;
    pi1.minLRPI      = .1;//////////////////////////
    pi1.maxCandidate = 10;
    pi1.minHRI       = 1;
    pi1.minHSI       = 6;
    pi1.minHIO       = 15;
    pi1.useVdsOnce   = 1;
    rcD0_BG1->addDaughter(pi1);

    attribute k1;
    k1.topp         = kinema::findTopp(-321);
    k1.mass         = k1.topp->mass;
    k1.minP         = -200;
    k1.maxP         = 0;
    k1.minPt        = 0.3;
    k1.minLRK       = .1;////////////////////////
    k1.maxLRPI      = .01; ////////////////////////////////////
    k1.maxCandidate = 10;
    k1.minHRI       = 1;
    k1.minHIO       = 15;
    k1.minHSI       = 6;
    k1.useVdsOnce   = 1;
    rcD0_BG1->addDaughter(k1);

    if (verb) rcD0_BG1->printDaughters();
  }
  rcD0_BG1->rc();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  D0_BG1 <<<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}


//
//
//
int
Dplus__Piminus_Piplus_Kplus_BG()
{
  int verb=0;
  
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  Dp_BG2 <<<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcDp_BG2) {
    cout<<"initializing Dplus bg"<<endl;
    rcDp_BG2=new rcX("bg_Dp", "D+ -> Pi-Pi+K+", verb);
    decay dm;
    dm.rlevel        = 0;         // 0=full, 1=semi, 2=mtra
    dm.fit           = 103;       // = 'long' decay
    dm.book_tables   = 1;         // 1/0
    dm.root_output   = 1;         // make root file or not
    dm.verb          = verb;      // printout level
    rcDp_BG2->setMother(dm);      // set
    vertexAttrib vdm;
    vdm.rlevel           = 2;       // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vdm.topp             = kinema::findTopp(411); // pointer to mother
    vdm.max_vertex_chi2  = 10.0;    // limit for vertex acceptance 
    vdm.verb             = verb;    // printout level
    vdm.min_mass         = DM_LO; // minimum mass
    vdm.max_mass         = DM_HI; // maximum mass
    vdm.allowed_v0_tracks = 1;     // exlude tracks assigned to v0
    rcDp_BG2->setVertex(vdm);       // set

    attribute pi1;
    pi1.topp         = kinema::findTopp(-211);
    pi1.mass         = pi1.topp->mass;
    pi1.minP         = -200;
    pi1.maxP         = 0;
    pi1.minPt        = 0.15;
    pi1.minLRPI      = .1;/////////////////////////
    pi1.maxCandidate = 10;
    pi1.minHRI       = 1;
    pi1.minHIO       = 15;
    pi1.minHSI       = 6;
    pi1.useVdsOnce   = 1;
    rcDp_BG2->addDaughter(pi1);

    attribute pi2;
    pi2.topp         = kinema::findTopp(211);
    pi2.mass         = pi2.topp->mass;
    pi2.minP         = 0;
    pi2.maxP         = 200;
    pi2.minPt        = 0.15;
    pi2.minLRPI      = .1;/////////////////////////
    pi2.maxCandidate = 10;
    pi2.minHRI       = 1;
    pi2.minHIO       = 15;
    pi2.minHSI       = 6;
    pi2.useVdsOnce   = 1;
    rcDp_BG2->addDaughter(pi2);

    attribute k1;
    k1.topp         = kinema::findTopp(321);
    k1.mass         = k1.topp->mass;
    k1.minP         = 0;
    k1.maxP         = 200;
    k1.minPt        = 0.3;
    k1.minLRK       = .1; ///////////////////////////////////
    k1.maxLRPI      = .01; ////////////////////////////////////
    k1.maxCandidate = 10;
    k1.minHRI       = 1;
    k1.minHIO       = 15;
    k1.minHSI       = 6;
    k1.useVdsOnce   = 1;
    rcDp_BG2->addDaughter(k1);

    if (verb) rcDp_BG2->printDaughters();
  }
  rcDp_BG2->rc();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  Dp_BG2 <<<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}


//
//
//
int
selectD0()
{
  ARTE_mark( "rcX_D 0", 0 );
  static int old_total1=0;
  static int old_total2=0;
  static int old_total3=0;
  static int old_total4=0;
  int ret=0;
  maximizeRTRA();
  rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1040 ), 0., 1.); // new event
  ArteTable<RTRA>::iterator d1=ArteTable<RTRA>::end();
  D0();
  ArteTable<RTRA>::iterator d2=ArteTable<RTRA>::end();
  if (rcD0_RC1&&rcD0_RC1->total>old_total1) { old_total1=rcD0_RC1->total; ret=1; }
  if (rcD0_RC2&&rcD0_RC2->total>old_total2) { old_total2=rcD0_RC2->total; ret=1; }
  if (rcDm_RC3&&rcDm_RC3->total>old_total3) { old_total3=rcDm_RC3->total; ret=1; }
  if (rcDp_RC4&&rcDp_RC4->total>old_total4) { old_total4=rcDp_RC4->total; ret=1; }
  if (ret) {
    rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1040 ), 1., 1.); // D found
    // -- fill mass
    for (ArteTable<RTRA>::iterator id=d1; id!=d2; ++id) {
      ArtePointer<RVER> rver=(*id)->ever;
      float mass=rver->mass;
      rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1041 ), mass, 1.);
    }
  }
#ifdef USE_ROOT_DD
  ////////////////D0__Piminus_Kminus_BG();
  ///////////////Dplus__Piminus_Piplus_Kplus_BG();
#endif
  ARTE_mark( "rcX_D 1", 0 );
  return ret;
}

//
//
//
void
stopD0()
{
  if (rcD0_MC1) delete rcD0_MC1;
  if (rcD0_MC2) delete rcD0_MC2;
  if (rcD0_RC1) delete rcD0_RC1;
  if (rcD0_RC2) delete rcD0_RC2;
  if (rcDm_RC3) delete rcDm_RC3;
  if (rcDp_RC4) delete rcDp_RC4;
  if (rcD0_BG1) delete rcD0_BG1;
  if (rcDp_BG2) delete rcDp_BG2;
}
