
// *****************************************************************************
//
// source:    Selection of Phi mesons using the following channels:
//               K+ K-
//            Returns:
//               1 upon success, 0 otherwise  
//
// dependences: RTRA bank filled 
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 04 Aug 2000 (DD) code polishing
// 06 Aug 2000 (DD) quality histograms
// 06 Sep 2000 (DD) File description added
// 17 Sep 2000 (DD) Check that tracks point to a v0
//
// *****************************************************************************

#include "reco/rcX.h"
#include "reco/rcXkinema.h"
#include "rhp.h"
#include "dq/DQ.h"


rcX *rcPhi1=0;
rcX *rcPhi2=0;

int Phi__Kplus_Kminus_MC();
int Phi__Kplus_Kminus_RECO();
int maximizeRTRA();

//
//
//
int
Phi()
{
  if (ArteTable<MTRA>::size()) {
    Phi__Kplus_Kminus_MC();
  }
  else {
    Phi__Kplus_Kminus_RECO();
  }
  return 0;
}

//
//
//
int
Phi__Kplus_Kminus_MC()
{
  int verb=0;

  if (verb) cout<<">>>>>>>>>>>>>>>>>>>>  Phi1 <<<<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcPhi1) {
    cout<<"initializing Phi"<<endl;
    rcPhi1=new rcX("Phi", "Phi -> K+K-", verb);
    decay phi;
    phi.fit           = 2;   // = short decay
    phi.book_tables   = 1;   // 1/0
    phi.rlevel        = 2;   // 0=full, 1=semi, 2=mtra
    phi.verb          = verb;// printout level
    rcPhi1->setMother(phi);  // set
    vertexAttrib vphi;
    vphi.rlevel             = 2;     // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vphi.topp               = kinema::findTopp(333); // pointer to mother
    vphi.momentum_flag      = 1;     // momentum in fit?
    vphi.change_track_rtra  = 1;
    vphi.max_vertex_chi2    = 7.0;  // limit for vertex acceptance 
    vphi.verb               = verb;  // printout level
    vphi.min_mass           = 0.98;
    vphi.max_mass           = 1.11;
    vphi.check_wire         = 0;
    rcPhi1->setVertex(vphi);      // set

    attribute k1;
    k1.topp=kinema::findTopp(-321);
    k1.mass=k1.topp->mass;
    k1.lund=-321;
    k1.mother=333;
    k1.minP=-200;
    k1.maxP=0;
    //k1.minHIO=15;
    rcPhi1->addDaughter(k1);

    attribute k2;
    k2.topp=kinema::findTopp(321);
    k2.mass=k2.topp->mass;
    k2.lund=321;
    k2.mother=333;
    k2.minP=0;
    k2.maxP=200;
    //k2.minHIO=15;
    rcPhi1->addDaughter(k2);

    if (verb) rcPhi1->printDaughters();
  }
  rcPhi1->rc();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>>>  Phi1 <<<<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}



/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////  REAL RECO   /////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

int
Phi__Kplus_Kminus_RECO()
{
  int verb=0;
  
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  rec Phi2 <<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcPhi2) {
    cout<<"initializing Phi"<<endl;
    rcPhi2=new rcX("rec_Phi", "Phi -> K+K", verb);
    decay phi;
    phi.rlevel        = 0;     // 0=full, 1=semi, 2=mtra
    phi.fit           = 102;   // = short decay
    phi.book_tables   = 1;     // 1/0
    phi.root_output   = 1;     // 1/0 
    phi.verb          = verb;  // printout level
    rcPhi2->setMother(phi);    // set
    vertexAttrib vphi;
    vphi.rlevel            = 2;     // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vphi.topp              = kinema::findTopp(333); // pointer to mother
    vphi.max_vertex_chi2   = 10.0;  // limit for vertex acceptance 
    vphi.verb              = verb;  // printout level
    vphi.min_mass          = 0.98;
    vphi.max_mass          = 1.11;
    vphi.check_wire        = 1;     // try to find v0/wire 
    vphi.require_v0_tracks = 1;     // all tracks from a primary vertex
    rcPhi2->setVertex(vphi);        // set

    attribute k1;
    k1.topp         = kinema::findTopp(-321);
    k1.mass         = k1.topp->mass;
    k1.minP         = -200;
    k1.maxP         = 0;
    k1.minPt        = 0.3;
    k1.minLRK       = .0001;
    k1.minHIO       = 18;
    k1.minHRI       = 1;
    k1.minHSI       = 6;
    k1.maxCandidate = 10;
    k1.useVdsOnce   = 1;
    rcPhi2->addDaughter(k1);

    attribute k2;
    k2.topp         = kinema::findTopp(321);
    k2.mass         = k2.topp->mass;
    k2.minP         = 0;
    k2.maxP         = 200;
    k2.minPt        = 0.3;
    k2.minLRK       = .0001;
    k2.minHIO       = 18;
    k2.minHRI       = 1;
    k2.minHSI       = 6;
    k2.maxCandidate = 10;
    k2.useVdsOnce   = 1;
    rcPhi2->addDaughter(k2);

    if (verb) rcPhi2->printDaughters();
  }
  rcPhi2->rc();
  if (verb) rcPhi2->printTracks();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  rec Phi2 <<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}



//
//
//
int
selectPhi()
{
  ARTE_mark( "rcX_Phi 0", 0 );
  static int old_total=0;
  int ret=0;
  maximizeRTRA(); // use highest likelihood
  rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1050 ), 0., 1.); // new event
  ArteTable<RTRA>::iterator phi1=ArteTable<RTRA>::end();
  Phi();
  ArteTable<RTRA>::iterator phi2=ArteTable<RTRA>::end();
  if (rcPhi2->total>old_total) { old_total=rcPhi2->total; ret=1; }
  if (ret) {
    rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1050 ), 1., 1.); // event selected
    // -- fill mass
    for (ArteTable<RTRA>::iterator iphi=phi1; iphi!=phi2; ++iphi) {
      ArtePointer<RVER> rver=(*iphi)->ever;
      float mass=rver->mass;
      rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1051 ), mass, 1.);
    }
  }
  ARTE_mark( "rcX_Phi 1", 0 );
  return ret;
}

//
//
//
void
stopPhi()
{
  if (rcPhi1) delete rcPhi1;
  if (rcPhi2) delete rcPhi2;
}
