#include "rich/vessel.h"
#include "rich/Losses.h"

RICHVessel *RICH_Vessel=0;
int RICHVessel::classCounter=0;
extern int RICHdebug;
// ==========================================================================

// ----------------------------------------------------------------------
//
//  trace through gas, geometrical losses + interactions with gas
//

int RICHVessel::trace(Photon* p,double path) {
  int ret=NO_LOSS;
  double l;

  if (ProtonBeamShroud().intersection(*p,&l) && l< path) { 
   path=l;
    ret=VES_LOSS_PBEAMSHROUD;
    }
  if (ElectronBeamShroud().intersection(*p,&l) && l< path) {
    path=l;
    ret=VES_LOSS_EBEAMSHROUD;
    }
  int ret1;
  if ((ret1=radiator->trace(p,path))) return ret1;
  return ret;
  }
 
// ----------------------------------------------------------------------
//
//  raytracing through vessel & photon detector window
//

int RICHVessel::raytrace(Photon *pp) {
  int ret;
  double path=1E37;

  Photon& p=*pp;
  if (p.n().z()>0) {
    ret=0;

    if (!plane_mirror->fullmirror_intersection(p,&path)&&path>=0) {
      if ((ret=trace(pp,path))) return ret;
      return VES_LOSS_PMI_BACKSIDE;
      }
    if ((ret=spherical_mirror->reflect(pp))) return ret;
    }

  if ((ret=plane_mirror->reflect(pp))) return ret;
  if ((ret=photondetector_window->trace(pp))) return ret;
  return NO_LOSS;  
  }

// ---------------------------------------------------------------------------
//
//  tracking to upper detector (to find the ring center in the reconstruction)
//
//   return values:
//
//  t.x() is the position on the back side of the photon detector window.
//  l is the distance between the hit position on the spherical mirror
//  to the actual position t.x().
//

void RICHVessel::track_upper(StraightLine* t,double *l) {
  spherical_mirror->upper_reflect(t);
  Vec3 x=t->x();
  plane_mirror->upper_reflect(t);
  *l=norm(t->x()-x);
  x=t->x();
  photondetector_window->upper_trace(t);
  *l+=norm(t->x()-x);
  }

// ---------------------------------------------------------------------------
//
//  tracking to lower detector (to find the ring center in the reconstruction)
//
//   return values:
//
//  t.x() is the position on the back side of the photon detector window.
//  l is the distance between the hit location on the spherical mirror
//  to the actual position t.x().
//

void RICHVessel::track_lower(StraightLine *t,double *l) {
  spherical_mirror->lower_reflect(t);
  Vec3 x=t->x();
  plane_mirror->lower_reflect(t);
  *l=norm(t->x()-x);
  x=t->x();
  photondetector_window->lower_trace(t);
  *l+=norm(t->x()-x);
  }

// ----------------------------------------------------------------------
//
//  trace through gas, only geometrical losses
//

int RICHVessel::traceonly(Photon* p,double path) {
  int ret=NO_LOSS;
  double l;
  if (ProtonBeamShroud().intersection(*p,&l) && l< path) { 
    path=l;
    ret=VES_LOSS_PBEAMSHROUD;
    }
  if (ElectronBeamShroud().intersection(*p,&l) && l< path) {
    path=l;
    ret=VES_LOSS_EBEAMSHROUD;
    }
  p->x()+=path*p->n();
  return ret;
  }

// ---------------------------------------------------------------------------
//  evaluate the geometrical losses only (to estimate the fluctuations
//  in the number of photons in the reconstruction)
//

int RICHVessel::geometry_trace(Photon *pp) {
  int ret;
  
  if ((ret=spherical_mirror->fullmirror_reflectonly(pp))) return ret;
  if ((ret=plane_mirror->fullmirror_reflectonly(pp))) return ret;

  if ((ret=photondetector_window->traceonly(pp))) return ret;
  return NO_LOSS;
  }

// ---------------------------------------------------------------------------
//
//  Determine the transmission off the mirrors for cutting in the number
//  of generated photons
//
//   mode =0:  determine only overall efficiency (to get spectral range only)
//   mode =1:  determine spectral range and max reflectivity in the range
//             and factorize this out of the reflectivities
//   mode =2:  determine the wavelength dependent reflectivities 
//             and factorize this out of the reflectivities
//

void RICHVessel::MaxEfficiency(Table& refl,int numpoint,int mode) {

  Table& smr=spherical_mirror->reflectivity();
  Table& pmr=plane_mirror->reflectivity();

  double lmin=smr.x(0);
  double lmax=smr.x(smr.numpoints()-1);
  if (lmin>pmr.x(0)) lmin=pmr.x(0);
  if (lmax<pmr.x(pmr.numpoints()-1)) lmax=pmr.x(pmr.numpoints()-1);

  double dl=(lmax-lmin)/numpoint;

// ---------------------------------------------------------------------
//  reflecitivity of spherical mirror

  int i;
  Table smrefl;
  for (i=0;i<numpoint;i++) {
    double l=lmin+i*dl;
    smrefl.append(l,spherical_mirror->reflectivity(l));
    }

  int r,c;
  for (r=0;r<spherical_mirror->numrow();r++) {
    for (c=0;c<spherical_mirror->numcol();c++) {
      SingleSphericalMirror* sm=spherical_mirror->SingleMirror(r,c);
      if (!sm) {
        fprintf(stderr,
          "RICHVessel::MaxEfficiency: inconsistent spherical mirror set up\n");
        Exit(1);
        }
      int i;
      for (i=0;i<numpoint;i++) {
        double l=lmin+i*dl;
        double r=sm->reflectivity().f(l);
        if (smrefl.y(i)<r) smrefl.y(i)=r;
        }
      }
    }

  if (mode!=0) {
    if (mode==1) {
      double effmax=0;
      int i;
      for (i=0;i<numpoint;i++) if (effmax<smrefl.y(i)) effmax=smrefl.y(i);
      for (i=0;i<numpoint;i++) if (smrefl.y(i)!=0)  smrefl.y(i)=effmax;
      }


    for (i=0;i<smr.numpoints();i++) {
      double l=smr.x(i);
      if (smrefl.f(l)>0) smr.y(i)/=smrefl.f(l);
      }
    int r,c;
    for (r=0;r<spherical_mirror->numrow();r++) {
      for (c=0;c<spherical_mirror->numcol();c++) {
        Table& smr=spherical_mirror->SingleMirror(r,c)->reflectivity();
        for (i=0;i<smr.numpoints();i++) {
          double l=smr.x(i);
          if (smrefl.f(l)>0) smr.y(i)/=smrefl.f(l);
          }
        }
      }
    }
    
// -------------------------------------------------------------------
//   same with plane mirrors
  
  Table pmrefl;
  for (i=0;i<numpoint;i++) {
    double l=lmin+i*dl;
    pmrefl.append(l,plane_mirror->reflectivity(l));
    }


  for (r=0;r<plane_mirror->numrow();r++) {
    for (c=0;c<plane_mirror->numcol();c++) {
      SinglePlaneMirror* pm=plane_mirror->SingleMirror(r,c);
      if (!pm) {
        fprintf(stderr,
          "RICHVessel::MaxEfficiency: inconsistent plane mirror set up\n");
        Exit(1);
        }
      int i;
      for (i=0;i<numpoint;i++) {
        double l=lmin+i*dl;
        double r=pm->reflectivity().f(l);
        if (pmrefl.y(i)<r) pmrefl.y(i)=r;
        }
      }
    }

  if (mode!=0) {
    if (mode==1) {
      double effmax=0;
      int i;
      for (i=0;i<numpoint;i++) if (effmax<pmrefl.y(i)) effmax=pmrefl.y(i);
      for (i=0;i<numpoint;i++) if (pmrefl.y(i)!=0) pmrefl.y(i)=effmax;
      }

    for (i=0;i<pmr.numpoints();i++) {
      double l=pmr.x(i);
      if (pmrefl.f(l)>0) pmr.y(i)/=pmrefl.f(l);
      }
    int r,c;
    for (r=0;r<plane_mirror->numrow();r++) {
      for (c=0;c<plane_mirror->numcol();c++) {
        Table& pmr=plane_mirror->SingleMirror(r,c)->reflectivity();
        for (i=0;i<pmr.numpoints();i++) {
          double l=pmr.x(i);
          if (pmrefl.f(l)>0) pmr.y(i)/=pmrefl.f(l);
          }
        }
      }
    }
    
// --------------------------------------------------------------------
//  total reflectivity

  for (i=0;i<numpoint;i++) {
    refl.append(smrefl.x(i),smrefl.y(i)*pmrefl.y(i));
    }  
  }


// ---------------------------------------------------------------------------
//
//  initialisation from ascii files
//

RICHVessel::RICHVessel(char *path) {
  if (RICH_Vessel)  memcpy(this,RICH_Vessel,sizeof(RICHVessel));
  else {
    //cout<<"RICHVessel::RICHVessel(char *path)"<<endl;

  if (path) {
    char n[256],s[256];
    strcpy(n,path);
    if (n[strlen(n)-1]!='/') strcat(n,"/");   
    
    strcat(n,"vessel.dat");
    ScriptFile f(n);
    while (f.gettag(n)) {
      f.skipline(s);
      if (!strcmp(n,"Vessel")) {
        if (RICHdebug){ printf("scanning vessel\n");fflush(stdout);}
        char n[256],tag[256];
        while (f.getline(n)) {
          char *s=n;
          sscanf(s,"%s",tag);s+=f.itemlength(s,1);
          if (RICHdebug) printf("vesselline %s tag   %s\n",n,tag);
          if (!strcmp(tag,"ProtonBeamShroud")) {
            pshroud=Tube(f);
            }
          else if (!strcmp(tag,"ElectronBeamShroud")) {
            eshroud=Tube(f);
            }
          else if (!strcmp(tag,"dielectric")) {
            radiator=new DiElectric(f);  
            }
          else if (!strcmp(tag,"end")) break;
          else {
            fprintf(stderr,
                 "vessel.dat:  Unknown tag: %s\n",tag);
            Exit(1);
            }
          }  
        }
      else {
        fprintf(stderr,"vessel.dat: Unknown tag: %s\n",n);
        Exit(1);
        }
      }
    CVES a;
    a.pori[0]=ProtonBeamShroud().x0().x();
    a.pori[1]=ProtonBeamShroud().x0().y();
    a.pori[2]=ProtonBeamShroud().x0().z();
    a.pez [0]=ProtonBeamShroud().ez().x();
    a.pez [1]=ProtonBeamShroud().ez().y();
    a.pez [2]=ProtonBeamShroud().ez().z();
    a.prad   =ProtonBeamShroud().radius();
    a.plen   =ProtonBeamShroud().length();
    a.eori[0]=ElectronBeamShroud().x0().x();
    a.eori[1]=ElectronBeamShroud().x0().y();
    a.eori[2]=ElectronBeamShroud().x0().z();
    a.eez [0]=ElectronBeamShroud().ez().x();
    a.eez [1]=ElectronBeamShroud().ez().y();
    a.eez [2]=ElectronBeamShroud().ez().z();
    a.erad   =ElectronBeamShroud().radius();
    a.elen   =ElectronBeamShroud().length();
    a.irad   =radiator->ArteTableIndex();
    ArteTable<CVES>::push_back(a);

    strcpy(n,path);
    if (n[strlen(n)-1]!='/') strcat(n,"/");   
    strcat(n,"PhotonDetectorEntryWindow.dat");
    ScriptFile fpdw(n);
    while (fpdw.gettag(n)) {
      fpdw.skipline(s);
      if (!strcmp(n,"PhotonDetectorEntryWindow")) {
        if (RICHdebug){ printf("scanning window\n");fflush(stdout);}
        photondetector_window=new PhotonDetectorEntryWindow(fpdw);
        }
      else {
        fprintf(stderr,"PhotonDetectorEntryWindow.dat: Unknown tag: %s\n",n);
        Exit(1);
        }
      }
 
    strcpy(n,path);
    if (n[strlen(n)-1]!='/') strcat(n,"/");   
    strcat(n,"SphericalMirror.dat");
    ScriptFile fsm(n);
    while (fsm.gettag(n)) {
      fsm.skipline(s);
      if (!strcmp(n,"SphericalMirror")) {
        if (RICHdebug){ printf("scanning spherical mirror\n");fflush(stdout);}
        spherical_mirror=new SphericalMirror(fsm);
        }
      else {
        fprintf(stderr,"SphericalMirror.dat: Unknown tag: %s\n",n);
        Exit(1);
        }
      }
       
      
    strcpy(n,path);
    if (n[strlen(n)-1]!='/') strcat(n,"/");   
    strcat(n,"PlaneMirror.dat");
    ScriptFile fpm(n);
    while (fpm.gettag(n)) {
      fpm.skipline(s);
      if (!strcmp(n,"PlaneMirror")) {
        if (RICHdebug){  printf("scanning plane mirror\n");fflush(stdout);}
        plane_mirror=new PlaneMirror(fpm);
        }
      else {
        fprintf(stderr,"PlaneMirror.dat: Unknown tag: %s\n",n);
        Exit(1);
        }
      }
    }
  else {  // init from Arte Tables

    ArtePointer<CVES> a(1);
    ProtonBeamShroud().x0()=Vec3(a->pori[0],a->pori[1],a->pori[2]);
    ProtonBeamShroud().ez()=Vec3(a->pez [0],a->pez [1],a->pez [2]);  
    ProtonBeamShroud().setrotation(ProtonBeamShroud().ez());
    ProtonBeamShroud().radius()=a->prad;
    ProtonBeamShroud().length()=a->plen;

    ElectronBeamShroud().x0()=Vec3(a->eori[0],a->eori[1],a->eori[2]);
    ElectronBeamShroud().ez()=Vec3(a->eez [0],a->eez [1],a->eez [2]);  
    ElectronBeamShroud().setrotation(ElectronBeamShroud().ez());
    ElectronBeamShroud().radius()=a->erad;
    ElectronBeamShroud().length()=a->elen;

    radiator=new DiElectric(ArtePointer<CDIM>(a->irad));

    spherical_mirror     = new SphericalMirror(ArtePointer<CSMI>(1));
    plane_mirror        = new PlaneMirror(ArtePointer<CPMI>(1));
    photondetector_window= new PhotonDetectorEntryWindow(ArtePointer<CPDW>(1));

    }
  }
} 

RICHVessel::~RICHVessel() {  
  delete spherical_mirror;
  delete plane_mirror;
  delete photondetector_window;
  delete radiator;
}

// ---------------------------------------------------------------------------
//
//  print to ascii files
//

void RICHVessel::print() {
  FILE *fp;  

  if (!(fp=fopen("vessel.dat","w"))) {
    fprintf(stderr,"RICHDetector::print: Can't open file: vessel.dat\n");
    Exit(1);
    }
  fprintf(fp,"#\nVessel\n");
  fprintf(fp,"#\n ProtonBeamShroud\n");
  ProtonBeamShroud().print(fp,0);
  fprintf(fp,"#\n ElectronBeamShroud\n");
  ElectronBeamShroud().print(fp,0);
  fprintf(fp,"#\n# Radiator\n");
  radiator->print(fp);
  fprintf(fp,"  end  # Vessel\n"); 
  fclose(fp);

 
  if (!(fp=fopen("PhotonDetectorEntryWindow.dat","w"))) {
    fprintf(stderr,"RICHDetector::print: Can't open file: vessel.dat\n");
    Exit(1);
    }
  if (photondetector_window) photondetector_window->print(fp);
  else {
    fprintf(stderr,"RICHDetector::print: photondetector_window not initialized\n");
    }
  fclose(fp);

  if (!(fp=fopen("SphericalMirror.dat","w"))) {
    fprintf(stderr,"RICHDetector::print: Can't open file: vessel.dat\n");
    Exit(1);
    }
  if (spherical_mirror) spherical_mirror->print(fp);
  else {
    fprintf(stderr,"RICHDetector::print: spherical_mirror not initialized\n");
    }
  fclose(fp);

  if (!(fp=fopen("PlaneMirror.dat","w"))) {
    fprintf(stderr,"RICHDetector::print: Can't open file: vessel.dat\n");
    Exit(1);
    }
  if (plane_mirror) plane_mirror->print(fp);
  else {
    fprintf(stderr,"RICHDetector::print: plane_mirror not initialized\n");
    }
  fclose(fp); 
  } 

