// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   21. Aug 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: HERA-B track covariance matrix class
//              Attention: Consturctor makes transformation
//              sigma(Q/p) -> sigma(p) (a Vt-requirement).
//
// changes:
// 21 Aug 2000 (TG) creation
// 06 Sep 2000 (TG) include VtUtil.hh added
// 06 Sep 2000 (TG) transform sigma(Q/p) into sigma(p) already in constructor
//
// *****************************************************************************

#include <iostream>
#include <assert.h>
#include "arte/arte.hh"
#include "vt++/CMatrix.hh"
#include "vt++/VtUtil.hh"

using namespace VERTEX;

namespace MATRIX {
  //============================================================================
  // CMatrix Constructors
  //============================================================================
  CMatrix::CMatrix(const CMatrix& rhs):VtSymMatrix(rhs) {}

  CMatrix::CMatrix(const ArtePointer<RTRA>& rtra):VtSymMatrix(5) {
    double p2   = sqr(1./rtra->pf);
    const int sign = sgn(rtra->pf);

    set_x      (rtra->cvf[0]);
    set_x_y    (rtra->cvf[1]);
    set_y      (rtra->cvf[2]);
    set_x_tx   (rtra->cvf[3]);
    set_y_tx   (rtra->cvf[4]);
    set_tx     (rtra->cvf[5]);
    set_x_ty   (rtra->cvf[6]);
    set_y_ty   (rtra->cvf[7]);
    set_tx_ty  (rtra->cvf[8]);
    set_ty     (rtra->cvf[9]);
    set_x_p    (rtra->cvf[10] * -sign * p2);
    set_y_p    (rtra->cvf[11] * -sign * p2);
    set_tx_p   (rtra->cvf[12] * -sign * p2);
    set_ty_p   (rtra->cvf[13] * -sign * p2);
    set_p      (rtra->cvf[14] * p2 * p2);

    // pinv might be 0 if no momentum is available for track
    if(x() <= 0. || y() <= 0. || tx() <= 0. || ty() <= 0. || p() < 0.) {
      cout << "CMatrix warning: corrupt covariance Matrix in RTRA " << rtra
	   << ": neg. or zero diagonal element!!!" << endl;
    }
  }

  CMatrix::CMatrix(const ArtePointer<RSEG>& rseg):VtSymMatrix(5) {
    double p2   = sqr(1./rseg->pf);
    const int sign = sgn(rseg->pf);

    set_x      (rseg->cvf[0]);
    set_x_y    (rseg->cvf[1]);
    set_y      (rseg->cvf[2]);
    set_x_tx   (rseg->cvf[3]);
    set_y_tx   (rseg->cvf[4]);
    set_tx     (rseg->cvf[5]);
    set_x_ty   (rseg->cvf[6]);
    set_y_ty   (rseg->cvf[7]);
    set_tx_ty  (rseg->cvf[8]);
    set_ty     (rseg->cvf[9]);
    set_x_p    (rseg->cvf[10] * -sign * p2);
    set_y_p    (rseg->cvf[11] * -sign * p2);
    set_tx_p   (rseg->cvf[12] * -sign * p2);
    set_ty_p   (rseg->cvf[13] * -sign * p2);
    set_p      (rseg->cvf[14] * p2 * p2);

    // pinv might be 0 if no momentum is available for track
    if(x() <= 0. || y() <= 0. || tx() <= 0. || ty() <= 0. || p() < 0.) {
      cout << "CMatrix warning: corrupt covariance Matrix in RSEG " << rseg
	   << ": neg. or zero diagonal element!!!" << endl;
    }

  }

  CMatrix::CMatrix(const ArtePointer<MTRA>& mtra):VtSymMatrix(5) {
    set_x      (1e-9);
    set_x_y    (0.);
    set_y      (1e-9);
    set_x_tx   (0.);
    set_y_tx   (0.);
    set_tx     (1e-9);
    set_x_ty   (0.);
    set_y_ty   (0.);
    set_tx_ty  (0.);
    set_ty     (1e-9);
    set_x_p    (0.);
    set_y_p    (0.);
    set_tx_p   (0.);
    set_ty_p   (0.);
    set_p      (1e-9);
  }

  //============================================================================
  // CMatrix::propagate: propagate Matrix by dz
  //============================================================================
  void CMatrix::propagate(const double dz) {
    set_x      ( x_prop(dz)    );
    set_x_y    ( x_y_prop(dz)  );
    set_x_tx   ( x_tx_prop(dz) );
    set_x_ty   ( x_ty_prop(dz) );
    set_x_p    ( x_p_prop(dz)  );
    set_y      ( y_prop(dz)    );
    set_y_tx   ( y_tx_prop(dz) );
    set_y_ty   ( y_ty_prop(dz) );
    set_y_p    ( y_p_prop(dz)  );

    // tx     is invariant
    // tx_ty  is invariant
    // ty     is invariant
    // tx_p   is invariant
    // ty_p   is invariant
    // p      is invariant
    return;
  }

} // namespace MATRIX
